﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Collections.ObjectModel;

namespace Kocic.RepositoryManagement.SystemFile
{
    /// <summary>
    /// Defers all entity object changes to the context and allows changes to be pushed to the context at once.
    /// </summary>
    /// <typeparam name="TContext">The object context to defer changes to.</typeparam>
    public class SystemFileContextDeferrer<TContext> : DataSourceDeferrer<TContext, LineObject>
        where TContext : FileContext, new()
    {
        /// <summary>
        /// Creates an instance of the SystemFileContextDeferrer class which uses a reference to the object context defined in the scope of a SystemFileExecutionContext instance.
        /// </summary>
        public SystemFileContextDeferrer()
            : base(SystemFileExecutionContext<TContext>.CurrentDataSource)
        {

        }

        /// <summary>
        /// Creates an instance of the EntityContextDeferrer class.
        /// </summary>
        /// <param name="context">An instance of the object context to which to defer changes to.</param>
        public SystemFileContextDeferrer(TContext context)
            : base(context)
        {
        }

        /// <summary>
        /// Disposes unmanaged resources.
        /// </summary>
        /// <param name="disposing">Value indicating if to invoke the method from the IDisposable.Dispose implementation or from the finalizer.</param>
        protected override void Dispose(Boolean disposing)
        {
            if (disposing)
            {
                if (this.Disposed)
                {
                    return;
                }

                try
                {
                    this.DataSource.SaveChanges();
                }
                finally
                {
                    DeferralOperation = null;

                    base.Dispose(disposing);
                }
            }
        }
    }
}
